#ifndef ANL_H
#define ANL_H
/*
   === definitions for analyzer structure ====================

   todo:
         - bananas
         - non-integer/non-equidistant bins
         - real/double cond. limits
         - errors
         - calibrations
         - text
         - linearizations
         - display attributes 
         - actual event data 
*/
#ifndef EXEC_TYPES_H
#include "exec/types.h"
#endif
#ifndef EXEC_LISTS_H
#include "exec/lists.h"
#endif
#ifndef EXEC_NODES_H
#include "exec/nodes.h"
#endif

#if defined(AMIGA)
#ifndef PROTO_EXEC_H
#include "proto/exec.h"
#endif
#endif
#include <limits.h>

/*
   --- constants -----------------------------------------------
*/
                        /* some system limits */
#define MANLNAME 18
#define MANLDIM   4
#define MANLCND 100     /* in fact a VSAM limit */
#define MANLPNT 100     /* in fact a VSAM limit */
#define MANLWDW 100     /* in fact a VSAM limit */
                        /* analyzer spectrum types */
#define ANLTYP_NONE    0
#define ANLTYP_BYTE    1
#define ANLTYP_SHORT   2
#define ANLTYP_LONG    4
#define ANLTYP_FLOAT  24
#define ANLTYP_DOUBLE 28

                        /* data element types (lDatEl member) */
#define ANLELM_NONE    0
#define ANLELM_ANL     1    /* analyzer */
#define ANLELM_LIN     2    /* linearization */
#define ANLELM_XCAL    3    /* x-axis calibration */
#define ANLELM_YCAL    4    /* y-axis calibration */

/*
   --- various status and action flags
*/
#define ANLSPC        0x00000080

#define ANLTEXT       0x00000100
#define ANLBAN        0x00000200
#define ANLCND        0x00000400
#define ANLDSP        0x00000800

#define ANLACT        0x00010000  /* analyzer active, force reset */
#define ANLON         0x00020000
#define ANLCLR        0x00040000
#define ANLDYN        0x00080000  /* dynamic creation (outside analysis) */
                                  /* misc. action flags */

#define _ANLPROMPT    0x00100000  /* print, reprompt */
#define _ANLDISP      0x00200000  /* call from display */
#define _ANLLIMINSIDE 0x00400000  /* window must be inside analyzer limits */ 
#define _ANLLIMI      0x00800000  /* analyzer limits given on input */

#define _ANLEXCLUSIVE 0x01000000  /* $ANAL() EXCLUSIVE */
#define _ANLNOCHECK   0x02000000  /* $ANAL() NOCHECK */
#define _ANLNOACCU    0x04000000  /* $ANAL() NOACCU */

#define  ANL_DEBUG    0x80000000
#define  ANL_VERBOSE  0x40000000
/*
   --- special (local) action flags
*/
#define AEXPGD_NORMBIN 0x00000001
/*
   --- basic structures -----------------------------------------------
*/
/*
   --- calibration structure per dimension
*/
#define MANLCALFCT  8 
#define MANLCALCOM 80
                                 /* calibration flag bits */
#define ANLCALNONE 0x00000000    /* no calibration */
#define ANLCALMEAN 0x00000001    /* explicit calibration of bin means */
#define ANLCALBORD 0x00000002    /* explicit calibration of bin borders */
#define ANLCALPOLY 0x00000004    /* calibration with polynomial */
#define ANLCALFUNC 0x00000008    /* calibration with user function  */

struct SANLCAL {
   unsigned long lFlag;          /* ANCAL* (none/mean/border/poly/func) */
//   int (*eFct)(void);          /* function name */
   int nPar;                     /* # of bins/borders/parameters */
   double *dPar;                 /* bins/borders/parameters */
   char cFct[MANLCALFCT+1];      /* function name */
   char cCom[MANLCALCOM+1];      /* calibration name */
};

/*
   --- banana condition structure
*/
#define MANLBANFCT 8

#define ANLBANNONE 0x00000000     /* no banana (rectangular cond.) */
#define ANLBANPOLY 0x00000001

#define ANLBANFUNC 0x00000004

struct SANLBAN {
   unsigned long lFlag;          /* none/poly/func */
//   int (*eFct)(void);          /* function name */
   int mPnt, nPnt;               /* max/current # of points */
   double *dPnt;                 /* banana points */   
   double dLim[MANLDIM][2];      /* lower/upper surrounding limits */
   char cFct[MANLBANFCT+1];      /* function name */
};

/*
   --- analyzer error structure
*/
#define MANLERRFCT 8             /* max. function name (MVS legacy) */
                                 /* slot indices for error structure */
enum { ANLERRU=0, ANLERRD, ANLERRL, ANLERRR, MANLERR };

#define ANLERRNONE 0x00000000
#define ANLERRSTAT 0x00000001
#define ANLERRBINS 0x00000002
#define ANLERRFUNC 0x00000004

struct SANLERR {
   unsigned long lFlag;          /* none/statistical/bins/func */
//   int (*eFct)(void);          /* function name */
   double *dErr;                 /* error data pointer */
   char cFct[MANLERRFCT+1];      /* function name */
};

/*
   --- limits for one dimension
*/
struct SANLLIM {
   int iLim[4];         /* lower/upper channel, bin size, # of bins */
};
                        /* conditions & flag tables
                           size = nCnd + 1 (for input condition)
                        */
struct SANLCND {
   unsigned long lEvent;/* # of events cond. was valid */
   unsigned long lFlag; /* cond. valid flag */
   unsigned long lAux;  /* help with condition check */
   unsigned long lUFlow, lOFlow;  /* # of counts below/above limits */
                        /* limits in four dimensions */
   struct SANLLIM sLim[MANLDIM];
   struct SANLBAN sBan;
};
                        /* display windows */
struct SANLWDW {
   char cWdw, cPad[3];
                        /* limits in four dimensions */
   struct SANLLIM sLim[MANLDIM];
};
                        /* display points */
struct SANLPNT {
   char cPnt, cPad[3];
                        /* limits in four dimensions */
   struct SANLLIM sLim[MANLDIM];
};
                        /* analyzer header */
struct SANL {
   char cAnl[MANLNAME], cPad[2];  /* name and padding */
   long lAnl, lEvt;               /* number and event number */
   int  iSpcType ;                /* spectrum type */
   int  nDim, nCnd, nWdw, nPnt;   /* dimension,
                                     # of conditions
                                       (in addition to input cond.),
                                     # of display windows,
                                     # of display points
                                  */
   long lDatEl;                   /* data element type (anal/lin/calib)*/
   unsigned long lFlag;           /* data element flag bits
                                     (active/exclusive/analon/dyn/...)
                                  */
   char cProc[8];                 /* name of analysis prog. */
   struct SANLCND *pCnd;          /* ptr. to contiguous condition data
                                     pCnd[0]   : input condition
                                     pCnd[1]   : 1st   condition
                                     pCnd[nCnd]: nth   condition
                                  */
   struct SANLWDW *pWdw;          /* ptr. to disp. windows */
   struct SANLPNT *pPnt;          /* ptr. to disp. points */
   void *pSpc;                    /* ptr. to spectrum */
   unsigned long lSpc;            /* # of spectrum elements (bins) */
   unsigned long lSpcUnit;        /* bytes per spectrum elements */
   unsigned long lSpcSpace;       /* bytes per spectrum */
   void *pPad[10];
   struct SANLERR sErr[MANLERR];  /* up,down,left,right error data */
   struct SANLCAL sCal[MANLDIM+1];/* x-cal each dim., y-cal. */
   double dActEvt[MANLDIM]; /* actual event data for double/float bins */
   long   lActEvt[MANLDIM]; /* actual event data for byte/short/long bins */
};
                      /* analyzer header arrays */

#define MAP 10        /* allocation increment */
struct SAP {
   long lAnlDisp;      /* last displayed analyzer */
   long lAnlTrace;     /* last traced analyzer    */
   int mAP, nAP;       /* max/current # of analyzers */
   struct SANL **pAP;  /* array of ptr.s to analyzer headers */
};

/*
   --- transfer structure (for VSAM interface)
*/
struct SANLXFER {
   unsigned long lFlag;
   unsigned long lSpec;           /* spectrum length (bytes) */
   void         *pSpec;           /* spectrum pointer */
   unsigned long lErr;            /* errors length (elements) */
   void         *pErr;            /* errors transfer area */
   unsigned long lCal;            /* calibration length (elements) */
   void         *pCal;            /* calibration pointer */
   struct SANLERR sErr[MANLERR];  /* error data structure */
   struct SANLCAL sCal[MANLDIM+1];/* calib data structure */
};
                                  /* transfer flags */
#define _ANLXFER_S 0x00000001     /* spectrum */
#define _ANLXFER_U 0x00000010     /* up errors */
#define _ANLXFER_D 0x00000020     /* down errors */
#define _ANLXFER_L 0x00000040     /* left errors */
#define _ANLXFER_R 0x00000080     /* right errors */
#define _ANLXFER_X 0x00000100     /* x-calibration */
#define _ANLXFER_A (_ANLXFER_S|_ANLXFER_U|_ANLXFER_D|_ANLXFER_L|_ANLXFER_R)

/*
   --- error codes ----------------------------------------------------
*/
#define ERRANL_SYS       -1
#define ERRANL_INVDAT    -2 /* Invalid data element type */
#define ERRANL_INVANL    -3 /* Invalid analyzer name */
#define ERRANL_DUPANL    -4 /* Duplicate analyzer name */
#define ERRANL_WRGANL    -5 /* Wrong analyzer name */
#define ERRANL_INVCND    -6 /* Invalid # of conditions */
#define ERRANL_INVSPC    -7 /* Invalid spectrum type */
#define ERRANL_INVLIM    -8 /* Invalid limits array */
#define ERRANL_INVDIM    -9 /* Invalid # of dimensions */
#define ERRANL_INVBIN   -10 /* invalid binsize */
#define ERRANL_INVEVT   -11 /* invalid event type */
#define ERRANL_NODEFEVT -12 /* no default event found */
#define ERRANL_CANTDEL  -13 /* Can't delete data element */
#define ERRANL_ANALACT  -14 /* analysis is running */
#define ERRANL_SYNTAX   -15 /* syntax error */
#define ERRANL_DIFFDIM  -16 /* dimensions differ */

#define ERRANL_DUPDAT   -17 /* duplicate data element */
#define ERRANL_NOSPACE  -20 /* no space for spectrum */
#define ERRANL_NOEVTNAM -21 /* event name not found */
#define ERRANL_NOEVTNUM -22 /* event number not found */
#define ERRANL_LGEVTNAM -23 /* event name too long */
#define ERRANL_NOANLNAM -24 /* analyzer name not found */
#define ERRANL_NOANLNUM -25 /* analyzer number not found */
#define ERRANL_LGANLNAM -26 /* analyzer name too long */
#define ERRANL_DIFFDAT  -27 /* data element types differ */
#define ERRANL_NOSPEC   -28 /* analyzer has no spectrum */
#define ERRANL_EVTDEF        -29 /* can't define event */
#define ERRANL_INVSHI        -30 /* invalid shift limits */
#define ERRANL_CONV          -31 /* conversion error */

#define ERRANL_INVBANMOD     -32 /* invalid polygon condition mode */
#define ERRANL_INVERRTYP     -33 /* invalid error type !(u,d,l,r) */
#define ERRANL_INVERRMOD     -34 /* invalid error mode !(stat,bins,func)*/
#define ERRANL_INVBANCNT     -35 /* invalid banana point count */
#define ERRANL_INVBANDIM     -36 /* wrong dimension for bananas */
#define ERRANL_INVCALPOL     -37 /* invalid calibration polynomial */
#define ERRANL_INVCALMOD     -38 /* invalid calibration mode */
#define ERRANL_INVCALDIM     -39 /* wrong dimension for calibration */
#define ERRANL_INVCALPAR     -40 /* invalid # parameters for calibration */

#define ERRANL_DUPINIT       -41
#define ERRANL_INIT          -42
#define ERRANL_CMDNOTYET     -43
#define ERRANL_PAREXCL       -44
#define ERRANL_INVPTR        -45
#define ERRANL_PARNOTYET     -46
#define ERRANL_INVWDW        -47
#define ERRANL_INVPNT        -48
#define ERRANL_VANLRH        -49
#define ERRANL_INVHEADLEN    -50
#define ERRANL_WRONGDATEL    -51
#define ERRANL_INVNOWDW      -52
#define ERRANL_INVNOPNT      -53
#define ERRANL_VANLRD        -54
#define ERRANL_VANLRE        -55
#define ERRANL_STATIC        -56
#define ERRANL_VSCGET        -57
#define ERRANL_VSCSET        -58
#define ERRANL_NOTINLIB      -59 
#define ERRANL_VANLD         -60

#define ERRANL_NOSUCHWDW     -61
#define ERRANL_NOSUCHPNT     -62
#define ERRANL_NOSUCHCND     -63

#define ERRANL_IMPORTFAILED  -64
#define ERRANL_NOFILE        -65

#define ERRANL_OPNOASSIGN    -66 
#define ERRANL_OPBADID       -67
#define ERRANL_OPNOOVERLAP   -68
#define ERRANL_OPBADLIMFIT   -69
#define ERRANL_OPBADBINFIT   -70
/*
   --- function definitions -------------------------------------------
*/
                                  /* command processors */
void _asccmd( char *, char *,
              char *, char *, char *, char *,
              long * );
void _aofcmd( char *, char *,
              long *, long * );
void _aoncmd( char *, char *,
              long *, long * );
void _aopcmd( char *, char *, char *, char *, char *, char *, char *, char *, 
              long * );
void _acstat( char *, char *,
              char *,
              long *, long *, long *,
              char *,
              long *, long * );
void _aexpcmd( char *, char *,
               char *, char *,
               long *, long *, long *, long *,
               long * );
void _aimpcmd( const char *, const char *,
               const char *, const char *, const char *,
               const char *,
               long * );
void _avlist( char *, char *, char *, char *,
              char *, char *, char *, long *
            );
void _avdel ( char *, char *, char *, char *,
              char *, char *, char *,
              long *, long *,
              long *
            );
void _avftch(
              char *, char *, char *,
              char *, char *, char *,
              char *, char *,
              long *, long *, long *, long *, long *,
              long *, long *, long *, long *, long *, long *,
              long *, long *, long *,
              long *
            );
void _aspace( long *);
void _amodc( char *, char *, char *, char *, char *,
             char *, char *, char *,
             long *,
             char *, char *, double *, double *,
             char *, char *, long *,
             long * );
void _awrcmd( char *, char *,
              char *, char *, char *,
              long *, long *, long *, long *,
              long *, long *, long *,
              long *, long *, long *,
              long *
            );
void _aclcmd( char *, char *, 
              long *, long *
            );
                                  /* exported routines */
int _acheck( char *, long *, char *, long *,
             long *, long *, long *, long [][4]);
int _achede( char *, long *, char *, long *,
             long *, long *, long *, long [][4], long * );
int _astcnd( long, long, long, long [MANLDIM][4] );
int _agtlim( long, long, char *, long, long [][4], unsigned long );
int _agticd( long, long, long [][4] );
int _agtcnd( long, long, long, long [][4] );
int _anattr( long, long, long *, long *, long *);
int _adattr( long, long, long *, long *, long *, long );
struct SANL *_anlptr( long, long, int * );
struct SANL *_adeptr( long, long, long *, int * );
struct SANL *_atrace( char *, long *, char *, long *, int * );
struct SANL *_atrade( char *, long *, char *, long *, long *, int * );
int _adatel( struct SANL *, long *, int );
int _agtspc ( long, long, long [][4], void *, long );
int _agtspca( long, long, long [][4], void **, long, long );
int _astspc ( long, long, long [][4], void *, long );
int _astspca( long, long, long [][4], void * , long, long );

                                  /* internal routines */
int _agtcal( long, long, long, struct SANLCAL *, unsigned long );
int _aptcal( long, long, long, struct SANLCAL *, unsigned long );
int _adlcal( struct SANL *, long,                   unsigned long );

int _agterr( long, long, long, struct SANLERR *, unsigned long );
int _apterr( long, long, long, struct SANLERR *, unsigned long );
int _adlerr( struct SANL *, long,                   unsigned long );

int _agtcban( struct SANL *, long, struct SANLBAN *, unsigned long );
int _aptcban( struct SANL *, long, struct SANLBAN *, unsigned long );
int _adlcban( struct SANL *, long,                   unsigned long );
int _adlban (                      struct SANLBAN *, unsigned long );
int _ashban( long, long, long, double [MANLDIM], unsigned long );
int _achban( struct SANLBAN *, double [2], unsigned long );

int _aclanl( struct SANL *, unsigned long );
int _adlanl( struct SANL *, unsigned long );
int _aspanl( struct SANL *, unsigned long );

int _aexpgd ( long, long, char *, long, long, unsigned long );
int _aexpgef( long, long, char *, unsigned long );

int _aninfo( long, long, char *);
int _adinfo( long, long, char *, long);
int _anldef( char *, long *, char *, long *,
             long, long, long, long [][4], long );
int _adedef( char *, long *, char *, long *,
             long, long, long, long [][4], long, long );
int _anldel( long, long );
int _adedel( long, long, long );
int _attall( long, long, long, long,
             int,
             long, long, long, long,
             int, char *, int );

int _aptclim( struct SANL *, long,   long [MANLDIM][4], unsigned long );
int _aptwlim( struct SANL *, char *, long [MANLDIM][4], unsigned long );
int _aptplim( struct SANL *, char *, long [MANLDIM][4], unsigned long );

int _agtclim( struct SANL *, long,   long [MANLDIM][4], unsigned long );
int _agtwlim( struct SANL *, char *, long [MANLDIM][4], unsigned long );
int _agtplim( struct SANL *, char *, long [MANLDIM][4], unsigned long );

int _aalcnd( struct SANL *, int, unsigned long );
int _afrcnd( struct SANLCND *, unsigned long );
int _aptcnd( struct SANL *, long, struct SANLCND *, unsigned long );
int _ac2cnd( struct SANL *, char *, struct SANLCND *, unsigned long );
int _acn2cnd(struct SANL *, char *, int, struct SANLCND *, unsigned long );

int _aalwdw( struct SANL *, int, unsigned long );
int _aalpnt( struct SANL *, int, unsigned long );

int _aspalc( long, long, long, long, long [][4] );

int _aprint( long, long, char *,
             int, int, int, int, int,
             char *, char *, long );

void _anlemsg( char *, int );
int _anlinit( void * );
int _anlterm(void);
int _anal( struct SANL *, unsigned long, ... );
int _areset( struct SAP * );

int _atrin( long *, long *, void *, short,
            struct SANLXFER *, long *, unsigned long );
int _atrout( long, long, void **, short *, struct SANLXFER *, unsigned long);

int _anlmod( char *, long *, char *, long *,
             long, long, long,
             long [][4], long [],
             unsigned long
           );
int _ademod( char *, long *, char *, long *,
             long, long, long,
             long [][4], long [],
             long,
             unsigned long
           );
int _amdcmd(
             char *, char *,
             char *, char *, char *, char *,
             char *, char *, char *, char *,
             char *,
             double, double, unsigned long
           );
int _amdnam(
             long, long,
             char *, long *,
             char *, long *,
             unsigned long
           );
int _amdspc(
             long, long,
             long [MANLDIM][4],
             double, double,
             unsigned long
           );
#endif
