#ifndef G_H
#define G_H
/*


   Copyright  (C) 1992-1997 M.Kraemer, D-64283 Darmstadt, Hoelgesstr. 16,
                             all rights reserved

   $Log: g.h,v $
   Revision 1.6  2005/12/16 11:03:04  kraemer
   MacOSX (darwin) support added

   Revision 1.5  2004/08/27 14:42:30  kraemer
   suppress banner

   Revision 1.4  2002/08/23 14:24:12  kraemer
   baseline

   Revision 1.3  1997/12/15 17:57:23  kraemer
   *** empty log message ***

   Revision 1.2  1997/08/29 11:36:32  kraemer
   major rework gfxattr,2dcuts

   Revision 1.1.1.1  1997/07/10 17:43:18  kraemer

 * Revision 1.1  1996/01/22  19:05:16  kraemer
 * Initial revision
 *
*/

#if defined(AMIGA)

#undef GFXX11
#undef GFXGPI
#define GFXAMIGA

#include <intuition/intuition.h>
#include <intuition/screens.h>
#include <graphics/gfx.h>
#include <graphics/gfxbase.h>
#include <graphics/gfxmacros.h>
#include <graphics/regions.h>
#include <graphics/text.h>
#include <proto/exec.h>
#include <proto/graphics.h>
#include <proto/layers.h>
#include <proto/intuition.h>
                             /* this is for 1.3 includes */
#ifndef GFXF_HR_AGNUS
#define ChipRevBits0 reserved[0] >> 24
#define GFXF_HR_AGNUS 1
#endif

#ifndef Y_H
#include "y.h"
#endif

typedef struct BitMap ABitMap;

#elif defined(__EMX__)

#undef GFXX11
#undef GFXAMIGA
#define GFXGPI

#ifndef Y_H
#include "y.h"
#endif

#include <os2.h>


typedef struct                 /* Amiga-Style bitmap */
{
    unsigned short   BytesPerRow;
    unsigned short   Rows;
    unsigned char   Flags;
    unsigned char  Depth;
    unsigned short pad;
    unsigned char *Planes[8];
} ABitMap;

#else

#undef GFXGPI
#undef GFXAMIGA
#define GFXX11

#if defined(__APPLE_CC__)
#include <limits.h>
#endif
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/cursorfont.h>

typedef struct                 /* Amiga-Style bitmap */
{
    unsigned short   BytesPerRow;
    unsigned short   Rows;
    unsigned char   Flags;
    unsigned char  Depth;
    unsigned short pad;
    unsigned char *Planes[8];
} ABitMap;

#endif

#ifndef U_H
#include "u.h"
#endif
#ifndef ILBM_H
#include "ilbm.h"
#endif

#include "polfit.h"
/*
   --- general flag layout
*/
#define   G_MASKGLOBAL 0xFF000000     /* global (e.g. debug) */
#define   G_MASKFUNC   0x00FF0000     /* function specific flags */
#define   G_MASKQUAL   0x0000FFFF     /* additional qualifiers for multiple functions */
/*
  --- header structure to keep info & list of graphics handles -------------
*/

struct SGHEAD {
   unsigned long lStatus;
#if defined(GFXGPI)
   int iPID;                           /* gfxserver process ID (OS/2) */
   HFILE hFile;                        /* communication pipe */
#endif
   struct List lh;
};
                    /* status flags */
                                       /* remember which library we opened */
#define GINIT_INTUITION 0x00000001
#define GINIT_DISKFONT  0x00000002
#define GINIT_LAYERS    0x00000004
#define GINIT_GFX       0x00000008
/*
   --- Image structure -----------------------------------------------------
*/
struct SGIMG {
   unsigned long lStatus;              /* type of image (IFF,XBM,...) */
   short iWidth, iHeight, iDepth, iPad;
   unsigned char *pData;               /* image data ptr. */
   struct SILBM sILBM;                 /* for IFF */
};
#define GIMG_IFF 0x00000001            /* an IFF image */
#define GIMG_XBM 0x00000002            /* an XBM image */
#define GIMG_ANY 0x0000000F            /* any image */
/*
   ------------- Strings with special characters and modifiers -------------
*/
struct SGSTR {
   int mStr;                           /* allocated length */
   char *pStr;                         /* ASCII + special chars */
   char *pMod;                         /* modifiers */
};

void _Gstrfr ( struct SGSTR * );
int  _Gstral ( int, struct SGSTR * );
int  _Gstrlen( struct SGSTR * );
int  _Gstrmodlen( struct SGSTR * );
int  _Gstrcat( struct SGSTR *, struct SGSTR * );
int  _Gstrdup2( struct SGSTR *, struct SGSTR *, struct SGSTR * );
int  _Gstrndup( struct SGSTR *, struct SGSTR *, int );
int  _Gstrdup ( struct SGSTR *, struct SGSTR * );
int  _Gstrsub ( struct SGSTR *, struct SGSTR *, int );
/*
   --- Graphics attribute structure ----------------------------------------
*/
struct SGFXATTR {
   double dHWidth;                    /* hetch width (cm) */
   double dLWidth;                    /* line width (cm) */
   double dCWidth;                    /* column width (cm) */
   double dRGB[3];                    /* colour composition (0..1) */
   double dBaseLine;                  /* histogram baseline */ 
   unsigned long lColour;             /* last used colour flags */
   unsigned long lFct;                /* gfx function (xor/copy/...) */
   unsigned long lHMode;              /* hetch mode */
   unsigned long lLMode;              /* line modes */
   short  iLWidth;                    /* line width (pixel) */
   short  iHWidth;                    /* hetch width (pixel) */
   short  iCWidth;                    /* column width (pixel) */
};
                                      /* line modes */
#define GFXATTR_LINEFULL  0x00000001
#define GFXATTR_LINEDASH  0x00000002
#define GFXATTR_LINEDOT   0x00000004
#define GFXATTR_LINENONE  0x00000008
#define GFXATTR_LINEANY   0x0000000F
                                      /* draw modes */
enum {
   GFXATTR_FCTCLR=0,                  /* 0 */
   GFXATTR_FCTAND,                    /* src AND dst */
   GFXATTR_FCTANDREVERSE,             /* src AND NOT dst */
   GFXATTR_FCTCOPY,                   /* src */
   GFXATTR_FCTANDINVERTED,            /* NOT src AND dst */
   GFXATTR_FCTNOOP,                   /* dst */
   GFXATTR_FCTXOR,                    /* src XOR dst (== complement) */
   GFXATTR_FCTOR,                     /* src OR dst */
   GFXATTR_FCTNOR,                    /* NOT src AND NOT dst */
   GFXATTR_FCTEQUIV,                  /* NOT src XOR dst */
   GFXATTR_FCTINVERT,                 /* NOT dst */
   GFXATTR_FCTORREVERSE,              /* src OR NOT dst */
   GFXATTR_FCTCOPYINVERTED,           /* NOT src */
   GFXATTR_FCTORINVERTED,             /* NOT src OR dst */
   GFXATTR_FCTNAND,                   /* NOT src OR NOT dst */
   GFXATTR_FCTSET                     /* 1 */
};
                                      /* hetch modes*/
enum { GFXATTR_HETCHNONE=0,
       GFXATTR_HETCHFULL,
       GFXATTR_HETCHVERT,
       GFXATTR_HETCHHORZ,
       GFXATTR_HETCHGRID,
       GFXATTR_HETCHD045,
       GFXATTR_HETCHD135,
       GFXATTR_HETCHDGRD,
       GFXATTR_HETCHPNTS 
};
#define GFXATTR_HETCHOPAQUE 0x00000080/* modifier flag for JAM2 mode */
                                      /* colour flags */
#define GFXATTR_FGP       0x00000001  /* foreground pen */
#define GFXATTR_BGP       0x00000002  /* background pen */
#define GFXATTR_GREYSCALE 0x00000004  /* greyscale (==don't normalize) */

                                      /* flag bits for function actions */
#define GGFXATTR_COLOUR   0x00010000
#define GGFXATTR_HMODE    0x00020000
#define GGFXATTR_LMODE    0x00040000
#define GGFXATTR_HWIDTH   0x00080000
#define GGFXATTR_LWIDTH   0x00100000
#define GGFXATTR_FCT      0x00200000
#define GGFXATTR_BASELINE 0x00400000
#define GGFXATTR_ALL ( GGFXATTR_COLOUR | GGFXATTR_HMODE | \
                       GGFXATTR_LMODE | GGFXATTR_HWIDTH | \
                       GGFXATTR_LWIDTH | GGFXATTR_FCT | \
                       GGFXATTR_BASELINE \
                     )


int _GGfxAttrInit   ( struct SGFXATTR *, unsigned long );
int _GGfxAttrInspect( struct SGFXATTR *, unsigned long );

/*
   --- Clipping 
*/
#define GCLP_INT     0x00100000        /* clipping at inner limits (axes) */
#define GCLP_EXT     0x00200000        /* clipping at outer limits (window) */
#define GCLP_ANY     0x00F00000        /* clip mask bits */

/*
   --- arc modes
*/
#define GARC_PIE     0x00020000        /* pie slice ellipse */
#define GARC_CHORD   0x00040000        /* chord ellipse */

#define GLINE_POLY  0x00020000         /* line should be closed */

/*
   --- colour modes
*/

/*
   --- histogram modes
*/
#define GHISTO_VERTICAL     0x00000001 /* histogram with vertical lines */

/*
   --- coord. transformations
*/
#define GTRAFO_LOG2CM 0x00000001
#define GTRAFO_CM2PHS 0x00000002
#define GTRAFO_CM2LOG 0x00000004
#define GTRAFO_PHS2CM 0x00000008
#define GTRAFO_ANY    0x0000000F
/*
   --- cursor stuff
*/
#define GCURSOR_CROSS  0x00000001  /* simple cross */
#define GCURSOR_HAIR   0x00000002  /* crosshair */
#define GCURSOR_CIRCLE 0x00000004  /* rubber circle */
#define GCURSOR_BAND   0x00000008  /* rubber band */
#define GCURSOR_RECT   0x00000010  /* rubber rectangle */
#define GCURSOR_XBAR   0x00000020  /* single bar to mark x */
#define GCURSOR_YBAR   0x00000040  /* single bar to mark y */
#define GCURSOR_OLD    0x00000100  /* flags old coordinates given */
#define GCURSOR_FRAME  0x00000200  /* additional frame */

/*
   --- Coordinate scale types
*/
enum {
   GSCALE_LIN=1, GSCALE_LOG, GSCALE_SQRT, GSCALE_RECIP
};
/*
   --- Miscellaneous values
*/
#define G_PSUNIT ( 2.54 / 72E0 )   /* PostScript unit (cm) */
#define G_PSLINEMAX 1024           /* max # of PostScript line pnts */
#define G_PSHETCHMIN (G_PSUNIT*2)  /* default PostScript hetch distance */
#define G_PSWIDTHMIN (G_PSUNIT/2)  /* default PostScript linewidth */
#define G_PSGAPMIN   (G_PSUNIT*4)  /* minimum PostScript dash gap */
#define G_PSDSHMIN   (G_PSUNIT*3)  /* minimum PostScript dash on */
#define G_PSMARGIN 1E0             /* PostScript printer margin (cm) */
#define G_PSPUNCH  1E0             /* PostScript punch   margin (cm) */
/*
   --- Plotfile stuff ------------------------------------------------
*/
enum { GPLOTFILE_AREA=1, GPLOTFILE_CURVE,   GPLOTFILE_COLOUR,
       GPLOTFILE_CLIP,   GPLOTFILE_ELLAREA, GPLOTFILE_ELLLINE,
       GPLOTFILE_LINE,   GPLOTFILE_OPNWDW,  GPLOTFILE_FREEWDW,
       GPLOTFILE_SYMBOL, GPLOTFILE_BELL,    GPLOTFILE_SCALE,
       GPLOTFILE_LWIDTH, GPLOTFILE_LMODE,
       GPLOTFILE_HWIDTH, GPLOTFILE_HMODE,
       GPLOTFILE_POINT, GPLOTFILE_FCT, GPLOTFILE_FLUSH
};

struct SPLOTFILEENTRY {
   int iTag, iLen;                 /* object type, total length (bytes) */
   unsigned long lFlag;
};
/*
   --- 2D cuts -------------------------------------------------------------
*/
                                      /* a single cut */
struct SG2DCUTVAL {
   double dVal;
   int nVal;                          /* # of 2D-values for this cut */
   struct SGFXATTR sGA;
};
                                      /* all cuts */
struct SG2DCUT {
   double dLim[2];                    /* min/max value of all cuts */
   unsigned long lMode;               /* presentation/data mode */
   int nCV;                           /* # of cut values */
   struct SG2DCUTVAL *psCV;           /* [nCV] */
};
                                      /* 2D modes */
#define G2D_MASKPMODE 0x0000000F      /* presentation: cluster, contour,...*/
#define G2D_MASKFMODE 0x000000F0      /* fill:         full,scatter */
#define G2D_MASKDMODE 0x00000F00      /* data:         histo,... */
#define G2D_MASKSMODE 0x0000F000      /* size:         linear,area */

#define G2D_FULL      0x00000010      /* full clusters/contours */
#define G2D_SCATTER   0x00000020      /* "pixel" clusters */
#define G2D_HISTO     0x00000100      /* assume a histogram */
#define G2D_SIZEAREA  0x00001000      /* cluster area prop. to cut value */

enum {
   G2D_CLUSTER=1,                     /* full clusters */
   G2D_CONTOUR,                       /* contour plot */
   G2D_ISOMETRIC                      /* isometric plot */
};

int _G2DCutInspect(      struct SG2DCUT *, unsigned long );
int _G2DCutInit   (      struct SG2DCUT *, unsigned long );
int _G2DCutFree   (      struct SG2DCUT *, unsigned long );
int _G2DCutLim    (      struct SG2DCUT *, unsigned long );
int _G2DCutAlloc  ( int, struct SG2DCUT *, unsigned long );
int _G2DCutDup    (      struct SG2DCUT *, struct SG2DCUT *, unsigned long );
int _G2DCutAscend (      struct SG2DCUT *, unsigned long );
int _G2DCutInterval( double, struct SG2DCUT *, unsigned long );

/*
   --- 2D windows (bananas) ------------------------------------------------
*/
struct SG2DWDW {
   double dLim[2][2];                 /* lower/upper limits in 2-dim */
   int nPnt;                          /* # of points */
   double *pdPnt;                     /* [nPnt*2] */
};

int _G2DWdwInspect(      struct SG2DWDW *, unsigned long );
int _G2DWdwInit   (      struct SG2DWDW *, unsigned long );
int _G2DWdwFree   (      struct SG2DWDW *, unsigned long );
int _G2DWdwLim    (      struct SG2DWDW *, unsigned long );
int _G2DWdwAlloc  ( int, struct SG2DWDW *, unsigned long );
int _G2DWdwDup    (      struct SG2DWDW *, struct SG2DWDW *, unsigned long );
int _G2DWdwTest   ( double [], struct SG2DWDW *, unsigned long );

/*
   --- Special flags for _GPoly()
*/
#define GPOLY_DATALIMITS 0x00010000
#define GPOLY_FITREPR    0x00100000
/*
   --- Special flags for _GArrow()
*/
#define GARROW_LINE  0x00100000
#define GARROW_ANGLE 0x00200000
/*
   --- Special flags for _GErrBar()
*/
#define GERRBAR_VERTICAL   0x00100000
#define GERRBAR_HORIZONTAL 0x00200000
/*
   --- Text stuff ----------------------------------------------------------
*/
/*
   --- a text node
*/
struct SGTEXT {
   double dXY[2], dDir[2], dExtent[4]; /* start, direction, max. extent */
   double dIncl;                       /* inclination of characters */
   double dSize;
   double dSpace, dLength;
   int iFont;
   unsigned long lFlag;
   struct SGSTR sGStr;                 /* text */
   struct SGFXATTR sGA;                /* gfx attributes */
};
#define GTEXT_EXTENT 0x00010000        /* text extent only, no drawing */

#define GT_EQUIFONT 0x00000001
#define GT_PROPFONT 0x00000002
                                /* character modifiers */
#define GT_MODALT  0x00000001
#define GT_MODSEP  0x00000001 /* dirty ! Works only if
                                 sub/supscripts are not to be separated ! */
#define GT_MODLO   0x00000002
#define GT_MODLOLO 0x00000003
#define GT_MODHI   0x00000004
#define GT_MODHIHI 0x00000005
#define GT_MODHILO 0x00000006
#define GT_MODLOHI 0x00000007
#define GT_MODSHFT 0x00000007

#define GT_MODOVR   0x00000008

#define GT_MODDENOMINATOR 0x00000010
#define GT_MODNUMERATOR   0x00000080
#define GT_MODLIMLO       0x00000020
#define GT_MODLIMUP       0x00000040

enum {
GT_80=0x80,
         GT_AE,GT_OE,GT_UE,GT_ae,GT_oe,GT_ue,GT_sz,
   GT_eg,GT_gr,GT_ce,GT_al,GT_be,GT_ga,GT_de,GT_ep,
/* 0x90 */
   GT_ze,GT_et,GT_th,GT_jo,GT_ka,GT_la,GT_my,GT_ny,
   GT_ks,GT_pi,GT_rh,GT_si,GT_ta,GT_yp,GT_ph,GT_ch,
/* 0xA0 */
   GT_ps,GT_om,GT_AL,GT_BE,GT_GA,GT_DE,GT_EP,GT_ZE,
   GT_ET,GT_TH,GT_JO,GT_KA,GT_LA,GT_MY,GT_NY,GT_KS,
/* 0xB0 */
   GT_PI,GT_RH,GT_SI,GT_TA,GT_YP,GT_PH,GT_CH,GT_PS,
   GT_OM,GT_hb,GT_lb,GT_in,GT_pd,GT_bl,GT_br,GT_if,
/* 0xC0 */
   GT_left,GT_right,GT_up,GT_down,GT_sb,GT_se,GT_mb,GT_me,
   GT__b,GT__e,GT_ap,GT_pr,GT_ge,GT_le,GT_appgt,GT_applt,
/* 0xD0 */
   GT_perp,GT_mult,GT_bull,GT_ident,GT_plm,GT_do,GT_ve,GT_te,
   GT_na,GT_pa,GT_1l,GT_subqu,GT_at,GT_li,GT_ic,GT_an,
/* 0xE0 */
   GT_dw,GT_ti,GT_TE,GT_VE,GT_DO,GT_EG,GT_GR,GT_ci,
   GT_CI,GT_Ce,

   GT_MAXCHAR
};

/*
   -------------------------- Axes --------------------------------------
*/
#define MGAXTICTABLE 9
                                   /* structure to describe tic marks */
struct SGAXTIC {
   unsigned long lFlag;
   unsigned long lScale;           /* lin/log/sqrt/recip/... */
   int nTicMod,                    /* large tic
                                      each nTicMod'th time */
       iTicMod;                    /* current mod counter
                                      0: start with a large tic */
   int nSubTicMod,                 /* large tic
                                      each nSubTicMod'th time */
       iSubTicMod;                 /* current small tic counter (for log)
                                   */
   int nTic;                       /* total # of tics */
   double dStep;                   /* external tic step, logical */
   double dLim[2];                 /* logical min/max */
   double dLen;                    /* axis length (cm) */
   double dDTV;                    /* logical units/cm */
   double dStartPos,               /* first tic pos (cm),   */
          dStepPos,                /* tic distance  (cm), */
          dStartNum,               /* first tic num,      */
          dStepNum;                /* tic distance        */
   double dTable[ MGAXTICTABLE ];  /* subtic table, for log,
                                      log10( 1..9 )
                                   */
};
                                   /* function prototypes */
int _GAxRecipTic(               struct SGAXTIC *, unsigned long );
int _GAxSqrtTic (               struct SGAXTIC *, unsigned long );
int _GAxLinTic  (               struct SGAXTIC *, unsigned long );
int _GAxLogTic  (               struct SGAXTIC *, unsigned long );
int _GAxSqrtTic (               struct SGAXTIC *, unsigned long );
int _GAxRecipTic(               struct SGAXTIC *, unsigned long );
                                   /* structure to describe axis */
struct SGAXIS {
   double dXD[2][2];               /* [0][*]: axis start (cm)
                                      [1][*]: axis direction (unit vector)
                                   */
   double dExtent[4];              /* before,after,below,above
                                   */
   struct SGTEXT sGT;              /* axis text */
   struct SGAXTIC sTic;            /* axis tic mark descriptor */
};
                                   /* structure to describe grid */
struct SGAXGRID {
   double dCorner[2][2];           /* [0][*]: lower left (cm)
                                      [1][*]: upper right (cm)
                                   */
   int iDim;                       /* grid dimension */
   int iPad;
   struct SGAXTIC sTic;            /* axis tic mark descriptor */
   struct SGFXATTR sGA;            /* gfx attributes */
};

enum { GAXIS_X=0, GAXIS_Y, GAXIS_Z };

#define GAXIS_EXTENT   0x00400000
#define GAXIS_MINUS    0x00200000      /* inverse axis */
#define GAXIS_TICMASK  0x0000000F
#define GAXIS_TICBELOW 0x00000001
#define GAXIS_TICABOVE 0x00000002
#define GAXIS_NUMMASK  0x000000F0
#define GAXIS_NUMBELOW 0x00000010
#define GAXIS_NUMABOVE 0x00000020
#define GAXIS_NUMPARA  0x00000040
#define GAXIS_NUMPERP  0x00000080
#define GAXIS_NUMREAL  0x00000100      /* axnums in float representation */

                                       /* function prototypes */
int _GAxNumFmt  ( double, unsigned long );
/*
   --- Graphics handle structure -------------------------------------------
*/
#define G_MAXDIM 2

struct SGH {
   struct Node ln;
   unsigned long lStatus;              /* global flags */
   int nDim;
   short  iSizeOfPhs, iPad;            /* size of physical coord. (bytes) */
   long lScale[3];                     /* coordinate scales (lin/log/etc).
                                          If < 0: inverse axis 
                                       */
   FILE *fpPlot;                       /* plotfile */
   FILE *fp;                           /* gfx output file */
                                       /* local coordinate trafos */
   unsigned long lLocalTrafo;          /* flag bits */
#define GLOCAL_SHIFTCM    0x00000001
#define GLOCAL_ROTATECM   0x00000002
#define GLOCAL_ROTATEAXIS 0x00000004
   double dLocalShift   [3];
   double dLocalAxis [2][3];
   double dLocalAngle[2];
                                       /* Limits
                                          [0/1/2/3] left/top/width/height */
   double dLim[4];                     /* plotfield, cm */
   double dClp[4];                     /* clip, cm */
   short  iLim[4];                     /* plotfield, pixel (screen only) */
   double dpix2cm[2];                  /* pixel per cm */
   double dLastPnt[G_MAXDIM];          /* last drawn point */
   double dLogScaled[G_MAXDIM][2];     /* logical limits, internally scaled */
   double dLog      [G_MAXDIM][2];     /* logical limits */
   double dcm       [4];               /* cm limits
                                          [0/1/2/3] left/low/width/height */
   short  icmPix    [4];               /* ditto, in pixels */
                                       /* coordinate trafos */
   double dLog2cm[2][3],
          dcm2Phs[2][3],
          dcm2Log[2][3],
          dPhs2cm[2][3];
#if defined(NEW)
   char *pcDev;                        /* gfx output device */
   char *pcFile;                       /* gfx output file */       
#endif
   struct SGFXATTR sGA;
   char cDev [PATH_MAX+1];             /* gfx output device */
   char cFile[PATH_MAX+1];             /* gfx output file */

#if defined(GFXX11)
   Window   xWdw;
   Display *pDsp;
   GC       xLGC;                      /* GC for line stuff */
   GC       xAGC;                      /* GC for area stuff */
   int iScreen;
   Pixmap pBitMap;                     /* for tile/stipple */
#endif

#if defined(GFXAMIGA)
   struct Screen *pScreen;
   struct Window *pWdw;
   struct YGC    *pGC;
   struct BitMap *pBM;                 /* for superbitmap wdw */
#endif

#if defined(GFXGPI)
   HFILE hFile;                        /* copy of gpi server pipe handle */ 
   HAB hab;                            /* copy of program-wide anchor block handle */
   HMQ hmq;                            /* copy of program-wide msg queue 
handle */
   HWND hwnd;                          /* window handle */
   HDC  hdc;                           /* device context */
   HPS  hps;                           /* presentation space */
#endif

};
/*
   --- global flag bits in "lStatus" field and
                           "lFlag" field for some functions
*/
#define G_COORDLOG   0x00000001        /* logical    coordinates */
#define G_COORDCM    0x00000002        /* centimeter coordinates */
#define G_COORDPHS   0x00000004        /* device     coordinates (pixel) */
#define G_COORDNORM  0x00000008        /* normalized coordinates (relative) */
#define G_COORDANY   0x000000FF

#define G_WDWPS      0x00000100        /* PostScript output */
#define G_WDWXFIG    0x00000200        /* xfig output */
#define G_WDWHPGL    0x00000400        /* HPGL output */
#define G_WDWIFF     0x00000800        /* IFF output */
#define G_WDWSCREEN  0x00001000        /* screen output */
#define G_WDWANY     0x0000FF00        /* any output */

#define G_MONOCHROME 0x01000000        /* monochrome output */
#define G_LAND       0x04000000        /* PORTrait  format for PS, == normal*/
#define G_PORT       0x02000000        /* LANDscape format for PS, == rotate*/
#define G_DONTPLOT   0x08000000        /* suppress plotfile output */

#define G_OVERSIZE   0x10000000        /* oversized windows allowed */
#define G_TEMPFILE   0x20000000        /* output = tmpfile, deletable */
#define G_PLOTFILE   0x40000000        /* plotfile active  */
#define G_NOBANNER   0x80000000        /* suppress banner page on printers */

                                       /* G_MASKGLOBAL flags */
#define G_NOMSG      0x40000000        /* suppress error messages */
#define G_DEBUG      0x80000000        /* global debug flag */

/*
   ------------------------ misc function prototypes -----------------------
*/

int _GGfxAttrGet( struct SGH *, struct SGFXATTR *, unsigned long );
int _GGfxAttrSet( struct SGH *, struct SGFXATTR *, unsigned long );

int _GAxRecip   ( struct SGH *, struct SGAXIS  *, unsigned long );
int _GAxSqrt    ( struct SGH *, struct SGAXIS  *, unsigned long );
int _GAxLin     ( struct SGH *, struct SGAXIS  *, unsigned long );
int _GAxLog     ( struct SGH *, struct SGAXIS  *, unsigned long );
int _GAxSqrt    ( struct SGH *, struct SGAXIS  *, unsigned long );
int _GAxRecip   ( struct SGH *, struct SGAXIS  *, unsigned long );
int _GAxGrid    ( struct SGH *, struct SGAXGRID  *, unsigned long );
int _GAxTxt     ( struct SGH *, struct SGAXIS *, struct SGTEXT *, int, unsigned long );
int _GAxNum     ( struct SGH *, struct SGAXIS *, struct SGTEXT *, double, double, unsigned long );

int _GTextMod( struct SGH *, struct SGTEXT *,
               double [4], double [2], double [4], double,
               int, unsigned long lFlag );
int _GText( struct SGH *, struct SGTEXT *, unsigned long lFlag );

int _G2DCluster( struct SGH *,
                 double [], double [], double [],
                 int, int,
                 double [3][2],
                 struct SG2DWDW *,
                 struct SG2DCUT *,
                 unsigned long
               );

int _G2DContour( struct SGH *,
                 double [], double [], double [],
                 int, int,
                 double [3][2],
                 struct SG2DWDW *,
                 struct SG2DCUT *,
                 unsigned long
               );

int _GInit( struct SGHEAD *, unsigned long );
int _GTerm( struct SGHEAD *, unsigned long );

void _GClsWdw( const char *, unsigned long );
void _GClsDev( const char *, unsigned long );
void _GFreeWdw( struct SGH *, unsigned long );
struct SGH *_GOpnWdw( const char *, const char *, void *, unsigned long, int * );
struct SGH *_GFndWdw( const char * );
int _GWdwIsScreen( struct SGH * );

int _GGetDev( struct SGH *, const char *, unsigned long );
int _GChkDev( struct SGH *, void *, unsigned long );

int _Gerrmsg( const char *, int );
char *_Gerrstr( int );

int PSOpnWdw   ( struct SGH *, unsigned long );
int PSClsWdw   ( struct SGH *, unsigned long );
int PSDrawLines( struct SGH *, double *, int, unsigned long );
int PSFillPolygon( struct SGH *, double *, int, unsigned long );
int PSFillArc  ( struct SGH *, double *, int, int );
int PSSymbol   ( struct SGH *, double *, double *, double, int, int, unsigned long );


int dummyOpnWdw( struct SGH *, unsigned long );
int dummyClsWdw( struct SGH *, unsigned long );

#if defined(GFXAMIGA)
int amiOpnWdw( struct SGH *, unsigned long );
int amiClsWdw( struct SGH *, unsigned long );
#endif

#if defined(GFXGPI)
int _gpiOpnWdw( struct SGH *, unsigned long );
int _gpiClsWdw( struct SGH *, unsigned long );
#endif

#if defined(GFXX11)

int x11OpnWdw( struct SGH *, unsigned long );
int x11ClsWdw( struct SGH *, unsigned long );
int x11DrawLines( struct SGH *, XPoint *, int, unsigned long );
int x11DrawArc  ( struct SGH *, XPoint *, int, int );
int x11FillArc  ( struct SGH *, XPoint *, int, int );
int x11Error( Display *, XErrorEvent *);

#endif

ABitMap *_GCreateABitMap( int, int, int );
void _GFreeABitMap( ABitMap * );

int ILBMopen( const char *, unsigned long, int *);
int ILBMread( int, void *, int, unsigned long );
int ILBMunpk( void *, struct SILBMBMHD *, ABitMap *, unsigned long );
int ILBMclos( int, unsigned long );
struct SGIMG *_GImgRead( const char *, unsigned long, int *);
int _GImgFree( struct SGIMG *, unsigned long );
int _GImgDisp( struct SGH *, struct SGIMG *, unsigned long );

int _GClpRct( struct SGH *, void *, unsigned long );

int _GEllArea( struct SGH *, void *, int, unsigned long );
int _GArea   ( struct SGH *, void *, int, unsigned long );
int _GHWidth ( struct SGH *, double, unsigned long );
int _GHMode  ( struct SGH *, unsigned long, unsigned long );

int _GEllLine( struct SGH *, void *, int, unsigned long );
int _GCurve  ( struct SGH *, void *, int, unsigned long );
int _GPoint  ( struct SGH *, void *, int, unsigned long );
int _GLine   ( struct SGH *, void *, int, unsigned long );
int _GVertLine(struct SGH *, void *, int, unsigned long );
int _GLWidth ( struct SGH *, double, unsigned long );
int _GLMode  ( struct SGH *, unsigned long, unsigned long );

int _GFlush  ( struct SGH *, unsigned long );

int _GSpline ( struct SGH *,
               double *, double *, double *,
               double *, double *, double *,
               double, int, unsigned long );
int _GPoly   ( struct SGH *,
               struct SPOLFITDATA *, struct SPOLFITPAR *,
               unsigned long );

int _GCoord( struct SGH *, void *, void *, int, unsigned long );
int _GScale( struct SGH *, double [][2], long [], int, unsigned long );

int _GWdwName( const char *, char *, int );
int _GSize( const char *, double [], unsigned long );
unsigned long _GOptSize( double [2], double [2], double [2], unsigned long );
int _GSymbol( struct SGH *, void *, double *, double, double,
              int, int, unsigned long lFlag );
int _GRamp  ( struct SGH *, void *, unsigned long, void *, int, int );
int _GBrkPnt( struct SGH *, unsigned long );
int _GCursor( struct SGH *, void *, void *, unsigned long, unsigned long );
struct SGH *_GCopy( struct SGH *, const char *, const char *, void *, unsigned long, int *);

int _GTrafo2 ( struct SGH *, unsigned long, double [2][3] );

int _GEllipse( struct SGH *, double *, unsigned long );
int _GFinger ( struct SGH *, double *, unsigned long );
int _GTsistor( struct SGH *, double *, unsigned long );
int _GDecay  ( struct SGH *, double *, unsigned long );
int _GCross  ( struct SGH *, double *, unsigned long );
int _GRect   ( struct SGH *, double *, unsigned long );
int _GDiode  ( struct SGH *, double *, unsigned long );
int _GRsistor( struct SGH *, double *, unsigned long );
int _GCircle ( struct SGH *, double *, unsigned long );
int _GScissor( struct SGH *, double *, unsigned long );
int _GArc    ( struct SGH *, double *, unsigned long );
int _GCapcty ( struct SGH *, double *, unsigned long );
int _GArrow  ( struct SGH *, double *, unsigned long );
int _GGrid   ( struct SGH *, double [2][2], unsigned long );

int _GHisto( struct SGH *,
             double *, double *, double *, double *,
             int, unsigned long
           );

int _GErrBar( struct SGH *,
              double *, double *, double *, double *,
              int, unsigned long
            );
/*
  --- error codes ----------------------------------------------------------
*/
#define ERRG_SYS             -1
#define ERRG_INIT            -2
#define ERRG_DUPINIT         -3 /* don't init twice */
#define ERRG_BADADDR         -4
#define ERRG_BADGFX          -5 /* bad gfx handle */
#define ERRG_BADCOORD        -6 /* bad coordinates, can't transform */
#define ERRG_BADWDWNAME      -7
#define ERRG_BADDEV          -8
#define ERRG_BADWDWLIM       -9
#define ERRG_BADWDWLIMRETRY -10 /* retry possible */ 
#define ERRG_WDWOPN         -11 
#define ERRG_NOCHIPMEM      -12
#define ERRG_BADCOORDTYPE   -13
#define ERRG_INVTRAFO       -14
#define ERRG_INVSCALE       -15
#define ERRG_INVDIM         -16
#define ERRG_INVLOGLIM      -17
#define ERRG_INVCMLIM       -18
#define ERRG_XOPENDSP       -19
#define ERRG_INVSIZE        -20
#define ERRG_BADTEXT        -21
#define ERRG_NOPLOTFILE     -22
#define ERRG_INVNPNT        -23
#define ERRG_2DINVNCUT      -24
#define ERRG_BADAXLEN       -25
#define ERRG_BADSHAPE       -26
#define ERRG_SPLNAT         -27
#define ERRG_INVNPAR        -28
#define ERRG_LSQPOL         -29
#define ERRG_BADIMGTYPE     -30
#define ERRG_ILBMINVCNT     -31 
#define ERRG_ILBMNOTFORM    -32
#define ERRG_ILBMNOTILBM    -33
#define ERRG_ILBMNOTBMHD    -34
#define ERRG_ILBMNOTCMAP    -35
#define ERRG_ILBMBADSIZE    -36
#define ERRG_BADXBM         -37
#define ERRG_BADBITMAPVAL   -38
#define ERRG_MATHDOMAIN     -39
#define ERRG_NUMFORMAT      -40
#define ERRG_DEVOPEN        -41
#define ERRG_DEVQUERY       -42
#define ERRG_DEVWDWOPEN     -43
#define ERRG_PSCREATE       -44
#define ERRG_DEVCAPS        -45
#define ERRG_2DEMPTY        -46
#define ERRG_2DWDWNPNT      -47
#endif

