#ifndef VSAM_H
#define VSAM_H
/*
   === declarations for VSAM subsystem ================================
*/
#ifndef Str
#define Str(v) #v
#endif
#ifndef XStr
#define XStr(v) Str(v)
#endif

#include <limits.h>

#ifndef _H_FCNTL 
#include "fcntl.h"
#endif
#ifndef _H_SETJMP
#include "setjmp.h"
#endif

#ifndef ANL_H
#include "anl.h"
#endif
#ifndef EVT_H
#include "evt.h"
#endif

#if defined(MVS) || defined(mvs)
#else
#include "gdbm.h"
#endif
/*
   --- VSAM master structure -------------------------------------------
*/

#define MVSAMUID 8
#define CBLANKUID "        "

#define MVSAMDIR PATH_MAX
#define MVSAMLIB PATH_MAX

//#define MVSAMDSN MVSAMDIR
//#define CBLANKDSN "        "

//#define MVSAMEXT MVSAMLIB
//#define CBLANKEXT "        "


#define MVSAMRUN 8
#define CALLRUN   "*       "
#define CBLANKRUN "        "

#define MVSAMDMP 4
#define CZERODMP   "   0"
#define CBLANKDMP  "    "
#define CALLDMP    "*   "

#define MVSAMDATE 8
#define MVSAMTIME 8
#define MVSAMTITLE 8
#define CBLANKTITLE "        "
#define MVSAMKEY  24
#define MVSAMDATA 4052

#define MVSAMANL 18
#define CBLANKANL "                  "
#define CALLANL   "*                 "
#define MVSAMEVT 18
#define CBLANKEVT "                  "
#define CALLEVT   "*                 "

#define MVSAMANLNO 4
#define CZEROANL  "   0"
#define MVSAMEVTNO 4
#define CZEROEVT  "   0"

                                       /* structure to overlay
                                          SVSAM* */
struct SVSAMPROTCOPY {                 /* based(sVSAM.pProt) */
  char cc[MVSAMDATA];
};
struct SVSAMCONTCOPY {                 /* based(sVSAM.pCont) */
  char cc[MVSAMDATA];
};

struct SVSAM {
   unsigned long lStatus;              /* global status */
   long lDmp;                          /* last ref. dump command number */
   long lDmpS, lEvtS, lAnlS;           /* saved values */
   long lProtS, lContS;                /* scan index for protocol/contents*/ 
   long lRec;                          /* record number */  
   void *pHead;                        /* ptr. to VSAM data header */
   void *pCont;                        /* ptr. to contents record */
   void *pProt;                        /* ptr. to protocol record */
   void *pProtS;
   void *pContS;
#if defined(MVS) || defined(mvs)
   FILE *fp;                           /* VSAM file ptr */
#else
   GDBM_FILE fp;                       /* GDBM file ptr */
#endif
   int  nProt;                         /* # of protocol records */
   int  nCont;                         /* # of contents records */
   int  nHead;                         /* size of header area */
#define MVSAMIO 4096
   void *pIO;                          /* I/O area */
   char *pFullName;
   char cUID[MVSAMUID],cUID0;
   char cDir[MVSAMDIR],cDir0;
   char cLib[MVSAMLIB],cLib0;
   char cRun[MVSAMRUN],cRun0;
                                       /* saved stuff */
   char cUIDS[MVSAMUID],cUIDS0;
   char cDirS[MVSAMDIR],cDirS0;
   char cLibS[MVSAMLIB],cLibS0;
   char cRunS[MVSAMRUN],cRunS0;
   char cDmpS[MVSAMDMP],cDmpS0;

   char cEvtS[MVSAMEVT],cEvtS0;
   char cAnlS[MVSAMANL],cAnlS0;
   jmp_buf sJmpBuf;
};
/*
   --- action and status flags ------------------------------------------
*/

#define VSAM_VALID     0x00000001      /* valid mask via VSCSET */
#define VSAM_ANLW      0x00000002      /* dump in progress */
#define VSAM_VFY       0x00000004      /* verify issued */

#define VSAM_PROTVALID 0x00000010      /* protocol is valid */
#define VSAM_PROTMOD   0x00000020      /* protocol has been modified */
#define VSAM_CONTVALID 0x00000040      /* copy of contents valid */
#define VSAM_CONTMOD   0x00000080      /* copy has benn verified */

#define VSAM_ALLOC     0x00000100      /* a VSAM lib is allocated */
#define VSAM_INP       0x00000200      /* a VSAM lib is opened for input */
#define VSAM_OUT       0x00000400      /* a VSAM lib is opened for output */
#define VSAM_UPD (VSAM_INP | VSAM_OUT)

#define VSAM_XFERU     0x00001000      /* transfer upper errors */
#define VSAM_XFERD     0x00002000      /* transfer lower errors */
#define VSAM_XFERL     0x00004000      /* transfer left errors */
#define VSAM_XFERR     0x00008000      /* transfer right errors */
#define VSAM_XFERX     0x00010000      /* transfer xcal */
#define VSAM_XFERS     0x00020000      /* transfer spectrum */
#define VSAM_XFER \
        (VSAM_XFERU|VSAM_XFERD|VSAM_XFERL|VSAM_XFERR|VSAM_XFERX|VSAM_XFERS)

#define VSAM_DEBUG     0x80000000      /* global debug */

/*
   --- contents record structure ---------------------------------------
*/
struct SVSAMCONT {
   short iVer;                         /* version number (def=1) */
   char  cUID [MVSAMUID];              /* user ID of owner (def=' ') */
   char  cDate[MVSAMDATE];
   char  cTime[MVSAMTIME];
   short iLen;                         /* used length of table */
                                       /*
                                       */
   struct SVSAMCTAB {
      char cEvtNam[MEVTNAME], cEvtNo[4],
           cAnlNam[MANLNAME], cAnlNo[4];
   } sCTab[2000];
};
                                       /* attributes of this version */
struct SVSAMCONTHEAD {
   short iVersion;                     /* version number */
   char  cTitle[8];                    /* title */
   short iType, iSubType;              /* type/subtype in datablk header*/
   short iInfo, iElm, mTab;            /* contents info w/o table,
                                          length of 1 table entry,
                                          max length of table */
};

/*
   --- protocol record structure ---------------------------------------
*/
struct SVSAMPROO {                     /* "old" version */
   short iVer;                         /* version number (def=2) */
   char  cUID[MVSAMUID];               /* user ID of owner (def=' ') */
   short iLen;                         /* used length of table */
                                       /*
                                       */
   struct SVSAMPTABO {
      char cRun[MVSAMRUN], cDmp[4], cDate[MVSAMDATE], cTime[MVSAMTIME];
   } sPTab[144];
};
struct SVSAMPROT {
   short iVer;                         /* version number (def=2) */
   char  cUID[MVSAMUID];               /* user ID of owner (def=' ') */
   short iLen;                         /* used length of table */
                                       /*
                                       */
   struct SVSAMPTAB {
      char cRun[MVSAMRUN], cDmp[MVSAMDMP];
   } sPTab[1000];
};
                                       /* attributes of this version */
struct SVSAMPROTHEAD {
   short iVersion;                     /* version number */
   char  cTitle[8];                    /* title */
   short iType, iSubType;              /* type/subtype of record
                                          (in data block header) */
   short mTab,                         /* max length of table */
         iInfo,                        /* protocol info w/o table */
         iElm;                         /* length of 1 table entry */
};
                                       /* attributes of "old" version */
struct SVSAMPROTHEADO {
   short iVersion;                     /* version number (=1) */
   char  cTitle[8];                    /* title */
   short iType, iSubType;              /* type/subtype of record (4/0)
                                          (in data block header) */
   short iMax,                         /* max # of concat. blks (=1) */
         iRun,                         /* current # of blk */
         mTab,                         /* max # of entries (=144) */
         iInfo,                        /* length of protocol info (=12) */
         iElm;                         /* length of element in table */
};
/*
   --- Data block header
*/
struct SVSAMBLOCK {
   short iByteCount; /* byte count for following data w/o headers */
   char  cBits;
   unsigned
   char  cType,      /* data type */
         cSpare2,    /* */
         cSubType,   /* data subtype */
         cMax,       /* max block # */
         cCur;       /* current block # */
   long  lCur;
};

#define VSAM_BITSPARE1     0xF8   /* 5 spare bits */
#define VSAM_BITORD        0x04   /* data reorganized */
#define VSAM_BITCMP        0x02   /* data compressed */
#define VSAM_BITDONTFORGET 0x01   /* non-forgettable bit */
/*
   --- VSAM <-> Analyzer transfer header(s)
*/

#define MVSAMWDW 100
#define MVSAMPNT 100
#define MVSAMCND 100

struct SVSAMTR1 {
   char c1[60];
   short nAnl; char cAnl[MVSAMANL];
   char c2[2];
   short iAnl;
   char c3[8];
   short nEvt; char cEvt[MVSAMEVT];
   char c4[2];
   short iEvt;
   short nHead;              /* length of transfer header (bytes) */
//#if defined(AMIGA)
//   long  lData;              /* length of data (bytes) */
//#else
   short iData[2];           /* for alignment ! */
//#endif
   short iSpcType, nDim, nCnd, nWdw, nPnt;
   long  lBin;               /* # of spectrum bins */
   long  lDatEl;             /* data element type */
};

struct SVSAMTR2 {
   char cc[15];
   char cWdw[MVSAMWDW];
};

struct SVSAMTR3 {
   char cc[15];
   char cPnt[MVSAMPNT];
};

struct SVSAMTR4 {
   char c1[4];
   char cDim[2];
   char c2[8];
   short iLim[4];
   short iDummy;
};

struct SVSAMTR5 {
   char cc[12];
   struct SVSAMTR5CND {
      short iLo, iHi;
   } sCnd[MVSAMCND];
};

struct SVSAMTR6 {
   char cc[12];
   struct SVSAMTR6WDW {
      short iLo, iHi;
   } sWdw[MVSAMWDW];
};

struct SVSAMTR7 {
   char cc[12];
   short iPnt[MVSAMPNT];
};

struct SVSAMTR8 {
   char cc[15];
   struct SVSAMTR8MSC {
      char cBan;
      char lValid[4];    /* originally: long, # cond was valid */
      char cDummy[4];
   } sMsc[MVSAMCND];
};

struct SVSAMTR9 {
   char cc[15];          /* TEXT: ' BAN. CND INFO:' */
   struct SVSAMTR9BAN {
      char cTyp[4];      /* TYPE OF CONDITION (RECT, POLN, OR FUNC) */
      char rRef[4];      /* orig: float,REF. VALUE FOR LINEARIZATION */
      char cT2[4];       /* FOR LATER USE */
      char cE1[8];       /* ENTRY NAME (ANALYSIS INTO CONDITION) */
      char cE2[8];       /* ENTRY NAME (DISPLAY OF 2-DIM. CUTS) */
      char iLen[2];      /* =0 ('RECT'), NO. OF POINTS ('POLN'),   
                            NO. OF PARAMETERS ('FUNC'), */
      char iL2[2];       /* FOR LATER USE */ 
   } sBan[MVSAMCND];
};

struct SVSAMTR10 {
   char cc[15];          /* TEXT: ' BAN. CND DATA:' */
   struct SVSAMTR10BAN {
      char rPnt[4];
   } sBan[32000];
};

struct SVSAMTR1X {
   char cc[15];          /* TEXT: ' xx ERROR DATA:' */
   char cTyp[4];         /* error type */
   char cT2[8];          /* for later */
   char cE1[8];          /* user program entry name */
};

struct SVSAMTR15 {
   char cc[15];          /* TEXT: ' X-CALIBRATION:' */
   char cT1[8];          /* for later */
   char cDescr[8];       /* calibration descriptor */
   char cT2[8];          /* for later */
   char cEntry[8];       /* user program entry name */
   char cCom[80];         /* axis description */
   char cP1[8];          /* for later */
};

/*
   --- function declarations --------------------------------------------
*/
int _vsminit( unsigned long );
int _vsmterm( unsigned long );
void _vsmemsg( char *, int );
void _vsmfatal( char * );

void _vcmdkill( char *, long *, char *, char *, long * );
void _vcmdfree( long * );

void _vkey( char [8], short, short, short, short, char [24] );
char *_vsmdsn( char *, char *, char * );
int _vinfo( char *, char *, char * );
int _valloc( char *, char *, char *, unsigned long );
int _vfree( unsigned long );
int _vclose( unsigned long );
int _vtclse( unsigned long );
int _vpcnv( void * );

int _vanlw( char *, char *, char *, char *, long *,
            char *, long, char *, long,
            void *, short, struct SANLXFER *, unsigned long
          );
int _vanld( char *, char *, char *, char *,
            long *, char *, long *, char *,
            long *, long
          );
int _vanlc( char *, char *, char *,
            char *, long *, char *, long *, char *, long *,
            long );
int _vanlrh( char *, long, long, long,
             void **, short *,
             unsigned long );
int _vanlrd( char *, long, long, long,
             void *, unsigned long,
             unsigned long );
int _vanlre( char *, long, long, long,
             void *, unsigned long,
             unsigned long );

int _vprotr( char *, long, unsigned long );
int _vprotu( char *, long *, unsigned long );
int _vprotg( unsigned long );
int _vprota( char *, long, unsigned long );
int _vprotd( char *, long, unsigned long );
int _vprotp( void **, unsigned long );
int _vprotw( unsigned long );

int _vcontr( char *, long, char *, long *, char *, long *, unsigned long );
int _vcontg( char *, long, unsigned long );
int _vconta( char *, long, char *, long *, char *, long *, unsigned long );
int _vcontd( char *, long, char *, long *, char *, long *, unsigned long );
int _vcontp( char *, long, void **, unsigned long );
int _vcontw( unsigned long );

int _vopsu( unsigned long );
int _vopsi( unsigned long );
int _vopso( unsigned long );

int _vaosi ( char *, char *, char *, unsigned long );
int _vaosu ( char *, char *, char *, unsigned long );
int _vaoso ( char *, char *, char *, unsigned long );
int _vaorp ( char *, char *, char *, char *, long, unsigned long );
int _vaorpu( char *, char *, char *, char *, long, unsigned long );
int _vaoup ( char *, char *, char *, char *, long *, unsigned long );
int _vaend ( unsigned long );

int _vprom( char *, char *, char *, unsigned long );

int _vexp( void *, int );
int _vexple( void *, int );
int _vcmp( void *, int );
int _vord( void *, int );
int _vreo( void *, int );
int _vrd( char [MVSAMKEY], char [MVSAMTITLE], 
          short *, short *, short *,   short *,  
          void  **, short *, 
          unsigned long *,
          unsigned long );
int _vwrt( char [MVSAMKEY], char *, 
           short, short, short, short,  
           void *, short, 
           unsigned long *,
           unsigned long );
int _vrwrt( char [MVSAMKEY], char *, 
           short, short, short, short,  
           void *, short, 
           unsigned long *,
           unsigned long );
int _vdel( char *,
           unsigned long *,
           unsigned long );
int _vscset( char *, char *, char *, char *,
             char *, char *, char *, unsigned long
           );
int _vscget( char *, char *, char *,
             long *, 
             char *, long *, char *, long *, unsigned long
           );
/*
   --- error flags ------------------------------------------------------
*/

#define ERRVSAM_SYS           -1
#define ERRVSAM_NOVSCSET      -2 /* no valid scan mask */
#define ERRVSAM_CONTENTSEMPTY -3 /* contents record empty */
#define ERRVSAM_CANTALLOC     -4 /* can't allocate VSAM dataset */
#define ERRVSAM_NOALLOC       -5 /* no VSAM dataset allocated */
#define ERRVSAM_GENCONT       -6 /* error generating contents record */
#define ERRVSAM_GENPROT       -7 /* error generating protocol record */
#define ERRVSAM_NOLIB         -8 /* no library specified */
#define ERRVSAM_BADLENGTH     -9 /* wrong data length */

#define ERRVSAM_FILESTAT     -10 /* file status incorrect */
#define ERRVSAM_INVARG       -11 /* invalid arguments */
#define ERRVSAM_ONCODE       -12 /* ON condition raised */
#define ERRVSAM_RECORDER     -13 /* incorrect order of records */
#define ERRVSAM_INVTYPE      -14 /* incorrect type/subtype */
#define ERRVSAM_INVDATATYPE  -15 /* incorrect data type */
#define ERRVSAM_MOREDATA     -16 /* more data found than expected */
#define ERRVSAM_LESSDATA     -17 /* less data found than expected */
#define ERRVSAM_PROTVAL      -18 /* no valid protocol copy */
#define ERRVSAM_INVHEADER    -19 /* invalid header, title or version */
#define ERRVSAM_DUPRUNDMP    -20 /* duplicate run-dump combination */
#define ERRVSAM_NORUNDMP     -21 /* run-dump combination not found */
#define ERRVSAM_RUNNODMP     -22 /* no dump found for run id */
#define ERRVSAM_RUNFULL      -23 /* run-dump table full */
#define ERRVSAM_KEYNOTFOUND  -24 /* key not found */
#define ERRVSAM_INVDUMPNO    -25 /* invalid dump # */
#define ERRVSAM_DUPANLEVT    -26 /* duplicate anl-evt combination */
#define ERRVSAM_NOANLEVT     -27 /* anl-evt combination not found */
#define ERRVSAM_CONV         -28 /* conversion error */
#define ERRVSAM_INVRUNDMP    -29 /* invalid run-dump combination */
#define ERRVSAM_ANLEVT       -30 /* anl-evt combination already exists */
#define ERRVSAM_ANLFULL      -31 /* analyzer table full */
#define ERRVSAM_GENDATETIME  -32 /* error generatin date & time */
#define ERRVSAM_PROTVER      -33 /* unknown protocol version */
#define ERRVSAM_NOANL        -34 /* data element not found */
#define ERRVSAM_GDBM         -35 /* error from gdbm */

#endif
